@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_auth.c,v 1.4 2005/06/25 00:58:46 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_auth.c,v 1.8 2006/02/05 01:59:27 devrim Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2005	Tatsuo Ishii
+ * Copyright (c) 2003-2006	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -133,18 +133,20 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	/* trust? */
 	if (pid == 0)
 	{
+		int msglen;
+
+		pool_write(frontend, "R", 1);
+
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			int msglen;
-
-			pool_write(frontend, "R", 1);
 			msglen = htonl(8);
 			pool_write(frontend, &msglen, sizeof(msglen));
-			msglen = htonl(0);
-			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
-			{
-				return -1;
-			}
+		}
+
+		msglen = htonl(0);
+		if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
+		{
+			return -1;
 		}
 		MASTER(cp)->auth_kind = 0;
 	}
@@ -198,55 +200,91 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	 * authentication ok. now read pid and secret key from the
 	 * backend
 	 */
-	kind = pool_read_kind(cp);
-	if (kind < 0)
+	for (;;)
 	{
-		return -1;
-	}
+		kind = pool_read_kind(cp);
+		if (kind < 0)
+		{
+			pool_error("pool_do_auth: failed to read kind before backendkeydata");
+			return -1;
+		}
+		else if (kind == 'K')
+			break;
 
-	/* error response? */
-	if (kind == 'E')
-	{
-		if (protoMajor == PROTO_MAJOR_V2)
-			ErrorResponse(frontend, cp);
-		else
-			SimpleForwardToFrontend(kind, frontend, cp);
-		return -1;
-	}
-	else if (kind != 'K')		/* BackendKeyData? */
-	{
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			/* process parameter status */
-			while (kind == 'S')
+			switch (kind)
 			{
-				if (ParameterStatus(frontend, cp) != POOL_CONTINUE)
+				case 'S':
+					/* process parameter status */
+					if (ParameterStatus(frontend, cp) != POOL_CONTINUE)
 						return -1;
+					pool_flush(frontend);
+					break;
 
-				pool_flush(frontend);
+				case 'N':
+					/* process notice message */
+					if (SimpleForwardToFrontend(kind, frontend, cp))
+						return -1;
+					pool_flush(frontend);
+					break;
 
-				kind = pool_read_kind(cp);
-				if (kind < 0)
-				{
-					pool_error("pool_do_auth: failed to read kind while processing ParamterStatus");
+					/* process error message */
+				case 'E':
+					SimpleForwardToFrontend(kind, frontend, cp);
+					pool_flush(frontend);
 					return -1;
-				}
+					break;
+
+				default:
+					pool_error("pool_do_auth: unknown response \"%c\" before processing BackendKeyData",
+							   kind);
+					return -1;
+					break;
 			}
 		}
 		else
 		{
-			pool_error("pool_do_auth: expect \"K\" got %c", kind);
-			return -1;
+			/* V2 case */
+			switch (kind)
+			{
+				case 'N':
+					/* process notice message */
+					if (NoticeResponse(frontend, cp) != POOL_CONTINUE)
+						return -1;
+					break;
+
+					/* process error message */
+				case 'E':
+					ErrorResponse(frontend, cp);
+					return -1;
+					break;
+
+				default:
+					pool_error("pool_do_auth: unknown response \"%c\" before processing V2 BackendKeyData",
+							   kind);
+					return -1;
+					break;
+			}
 		}
 	}
 
 	/*
 	 * message length (V3 only)
 	 */
-	if (protoMajor == PROTO_MAJOR_V3 && (length = pool_read_message_length(cp)) != 12)
+	if (protoMajor == PROTO_MAJOR_V3)
 	{
-		pool_error("pool_do_auth: invalid messages length(%d) for BackendKeyData", length);
-		return -1;
+		if (kind != 'K')
+		{
+			pool_error("pool_do_auth: expect \"K\" got %c", kind);
+			return -1;
+		}
+
+		if ((length = pool_read_message_length(cp)) != 12)
+		{
+			pool_error("pool_do_auth: invalid messages length(%d) for BackendKeyData", length);
+			return -1;
+		}
 	}
 
 	/*
@@ -314,18 +352,20 @@ int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 
 	if (status == 0)
 	{
+		int msglen;
+
+		pool_write(frontend, "R", 1);
+
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			int msglen;
-
-			pool_write(frontend, "R", 1);
 			msglen = htonl(8);
 			pool_write(frontend, &msglen, sizeof(msglen));
-			msglen = htonl(0);
-			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
-			{
-				return -1;
-			}
+		}
+
+		msglen = htonl(0);
+		if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
+		{
+			return -1;
 		}
 	}
 	else
@@ -345,6 +385,7 @@ static POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key
 	char kind;
 	int len;
 
+#ifdef NOT_USED
 	if (protoMajor == PROTO_MAJOR_V2)
 	{
 		/* return "Authentication OK" to the frontend */
@@ -356,6 +397,7 @@ static POOL_STATUS pool_send_auth_ok(POOL_CONNECTION *frontend, int pid, int key
 			return -1;
 		}
 	}
+#endif
 
 	/* send backend key data */
 	kind = 'K';
@@ -500,13 +542,18 @@ static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *fro
 	/* if authenticated, save info */
 	if (!reauth && kind == 0)
 	{
-		if (!backend->issecondary_backend && protoMajor == PROTO_MAJOR_V3)
+		if (!backend->issecondary_backend)
 		{
 			int msglen;
 
 			pool_write(frontend, "R", 1);
-			msglen = htonl(8);
-			pool_write(frontend, &msglen, sizeof(msglen));
+
+			if (protoMajor == PROTO_MAJOR_V3)
+			{
+				msglen = htonl(8);
+				pool_write(frontend, &msglen, sizeof(msglen));
+			}
+
 			msglen = htonl(0);
 			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
 			{
@@ -664,19 +711,22 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int rea
 	/* if authenticated, save info */
 	if (!reauth && kind == 0)
 	{
+		int msglen;
+
+		pool_write(frontend, "R", 1);
+
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			int msglen;
-
-			pool_write(frontend, "R", 1);
 			msglen = htonl(8);
 			pool_write(frontend, &msglen, sizeof(msglen));
-			msglen = htonl(0);
-			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
-			{
-				return -1;
-			}
 		}
+
+		msglen = htonl(0);
+		if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
+		{
+			return -1;
+		}
+
 		backend->auth_kind = 4;
 		backend->pwd_size = ntohl(size) - 4;
 		memcpy(backend->password, password, backend->pwd_size);
@@ -829,19 +879,22 @@ static int do_md5(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int reaut
 	/* if authenticated, save info */
 	if (!reauth && kind == 0)
 	{
+		int msglen;
+
+		pool_write(frontend, "R", 1);
+
 		if (protoMajor == PROTO_MAJOR_V3)
 		{
-			int msglen;
-
-			pool_write(frontend, "R", 1);
 			msglen = htonl(8);
 			pool_write(frontend, &msglen, sizeof(msglen));
-			msglen = htonl(0);
-			if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
-			{
-				return -1;
-			}
 		}
+
+		msglen = htonl(0);
+		if (pool_write_and_flush(frontend, &msglen, sizeof(msglen)) < 0)
+		{
+			return -1;
+		}
+
 		backend->auth_kind = 5;
 		backend->pwd_size = ntohl(size) - 4;
 		memcpy(backend->password, password, backend->pwd_size);
@@ -861,26 +914,26 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 	status = pool_read(MASTER(cp), &length, sizeof(length));
 	if (status < 0)
 	{
-		pool_error("read_message_length: error while reading message length");
+		pool_error("pool_read_message_length: error while reading message length");
 		return -1;
 	}
 	length = ntohl(length);
 
-	pool_debug("read_message_length: lenghth: %d", length);
+	pool_debug("pool_read_message_length: lenghth: %d", length);
 
 	if (DUAL_MODE)
 	{
 		status = pool_read(SECONDARY(cp), &length1, sizeof(length1));
 		if (status < 0)
 		{
-			pool_error("read_message_length: error while reading message length from secondary backend");
+			pool_error("pool_read_message_length: error while reading message length from secondary backend");
 			return -1;
 		}
 		length1 = ntohl(length1);
 
 		if (length != length1)
 		{
-			pool_error("read_message_length: length does not match between backends master(%d) secondary(%d)",
+			pool_error("pool_read_message_length: length does not match between backends master(%d) secondary(%d)",
 					   length, length1);
 			return -1;
 		}
@@ -888,7 +941,7 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 
 	if (length < 0)
 	{
-		pool_error("read_message_length: invalid message length (%d)", length);
+		pool_error("pool_read_message_length: invalid message length (%d)", length);
 		return -1;
 	}
 
@@ -909,26 +962,26 @@ int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
 	status = pool_read(MASTER(cp), &length, sizeof(length));
 	if (status < 0)
 	{
-		pool_error("read_message_length2: error while reading message length");
+		pool_error("pool_read_message_length2: error while reading message length");
 		return NULL;
 	}
 	length = ntohl(length);
 
-	pool_debug("read_message_length2: master lenghth: %d", length);
+	pool_debug("pool_read_message_length2: master lenghth: %d", length);
 
 	if (DUAL_MODE)
 	{
 		status = pool_read(SECONDARY(cp), &length1, sizeof(length1));
 		if (status < 0)
 		{
-			pool_error("read_message_length2: error while reading message length from secondary backend");
+			pool_error("pool_read_message_length2: error while reading message length from secondary backend");
 			return NULL;
 		}
 		length1 = ntohl(length1);
 
 		if (length != length1)
 		{
-			pool_debug("read_message_length2: length does not match between backends master(%d) secondary(%d)",
+			pool_debug("pool_read_message_length2: length does not match between backends master(%d) secondary(%d)",
 					   length, length1);
 		}
 	}
@@ -939,7 +992,7 @@ int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
 
 	if (length < 0 || length1 < 0)
 	{
-		pool_error("read_message_length2: invalid message length (%d) length1 (%d)", length, length1);
+		pool_error("pool_read_message_length2: invalid message length (%d) length1 (%d)", length, length1);
 		return NULL;
 	}
 
